/*
 * brute_force_solver.cpp
 *
 *  Created on: May 27, 2010
 *      Author: leden
 */

#include "brute_force_solver.h"

#include "brute_force_selector.h"
#include "seller.h"
#include "constraint.h"
#include "buyer_request.h"
#include "entity_io_utils.h"
#include "string_utils.h"
#include "purchasable.h"

#include <vector>
#include <iostream>

using namespace std;

namespace brute_force {

bool solve(Market& market, const std::string& filename,
		Price price_limit) {

	NoConstraint<Seller> seller_constr;
	NoConstraint<Selection> selection_constr;
	vector<BuyerRequest> reqs;

	if(!EntityIOUtils::loadRequests(reqs, filename)) {
		cerr << "Error loading requests" << endl;
	}

	cout << StringUtils::to_string(reqs.begin(), reqs.end(), ",") << endl;

	BruteForceSelector bfs(market, PurchasableData::instance());
	bfs.set_worst_acceptable_price(price_limit);
	if(bfs.start(reqs, seller_constr, selection_constr)) {
		cout << "Found solution: \n" << endl;
		cout << bfs.best_selection() << endl;
		return true;
	}
	else {
		cerr << "Failed to find solution with brute force!" << endl;
	}

	return false;
}

}
