/*
 * buyer_request.cpp
 *
 *  Created on: May 26, 2010
 *      Author: leden
 */

#include "buyer_request.h"

BuyerRequest::BuyerRequest(Constraint<Purchasable>* purchasable_constraint,
		Quantity quantity, bool disjunctive)
: constraint_(purchasable_constraint), quantity_(quantity), met_count_(0),
  disjunctive_(disjunctive) {
}

BuyerRequest::~BuyerRequest() {
	// TODO Auto-generated destructor stub
}

const Constraint<Purchasable>& BuyerRequest::constraint() const {
	return *constraint_.get();
}

Quantity BuyerRequest::quantity() const {
	return quantity_;
}

bool BuyerRequest::isDisjunctive() const {
	return disjunctive_;
}

bool BuyerRequest::incrementMetCount(int inc) {
	return set_met_count(met_count() + inc);
}

bool BuyerRequest::decrementMetCount(int dec) {
	return set_met_count(met_count() - dec);
}

bool BuyerRequest::set_met_count(int met_count) {
	bool was_met = isMet();
	met_count_ = met_count;
	return was_met ^ isMet();
}

int BuyerRequest::met_count() const {
	return met_count_;
}

bool BuyerRequest::isMet() const {
	return met_count() >= quantity();
}

int BuyerRequest::remainingUntilMet() const {
	if(isMet()) return 0;
	return quantity() - met_count();
}

std::ostream& operator<<(std::ostream& os, const BuyerRequest& br) {
	os << "Request #" << br.id() << ": " << br.name();
	os << "(met " << br.met_count() << "/" << br.quantity() << ")" << endl;
	return os;
}
