/*
 * ga_solver.cpp
 *
 *  Created on: May 30, 2010
 *      Author: leden
 */

#include "ga_solver.h"

#include <ecf/ECF.h>

#include "ga_selector.h"
#include "seller.h"
#include "constraint.h"
#include "buyer_request.h"
#include "entity_io_utils.h"
#include "string_utils.h"

#include <vector>
#include <iostream>

using namespace std;


namespace ga {

bool solve(Market& market, const string& requests_filename,
		const std::string& param_file, const std::string& argv0,
		Price price_limit) {

	NoConstraint<Seller> seller_constr;
	NoConstraint<Selection> selection_constr;
	vector<BuyerRequest> reqs;

	if(!EntityIOUtils::loadRequests(reqs, requests_filename)) {
		cerr << "Error loading requests" << endl;
		return false;
	}

	EntityIOUtils::printMarket(market);

	cout << StringUtils::to_string(reqs.begin(), reqs.end(), ",") << endl;

//	BruteForceSelector bfs(market, PurchasableData::instance());
//	if(bfs.start(reqs, seller_constr, selection_constr)) {
//		cout << "Found solution: \n" << endl;
//		cout << bfs.best_selection() << endl;
//	}
//	else {
//		cerr << "Failed to find solution with BF-GA!" << endl;
//	}
//
//	return ;

	GASelector gas(market, PurchasableData::instance(),
			param_file, argv0);
	gas.set_worst_acceptable_price(price_limit);
	if(gas.start(reqs, seller_constr, selection_constr)) {
		cout << "Found solution: \n" << endl;
		cout << gas.best_selection() << endl;
		return true;
	}
	else {
		cerr << "Failed to find solution with GA!" << endl;
	}
	return false;
}


}
