/*
 * greedy_solver.cpp
 *
 *  Created on: May 31, 2010
 *      Author: leden
 */

#include "greedy_solver.h"

#include "greedy_selector.h"
#include "seller.h"
#include "constraint.h"
#include "buyer_request.h"
#include "entity_io_utils.h"
#include "string_utils.h"
#include "flow_based_greedy_selector.h"

#include <iostream>
#include <vector>

using namespace std;

namespace greedy {

void solve2(Market& market, const std::string& filename, Price price_limit) {
	NoConstraint<Seller> seller_constr;
	NoConstraint<Selection> selection_constr;
	vector<BuyerRequest> reqs;

	if(!EntityIOUtils::loadRequests(reqs, filename)) {
		cerr << "Error loading requests" << endl;
	}

	cout << StringUtils::to_string(reqs.begin(), reqs.end(), ",") << endl;

	FlowBasedGreedySelector fbgs(market, PurchasableData::instance());
	fbgs.set_worst_acceptable_price(price_limit);
	if(fbgs.start(reqs, seller_constr, selection_constr)) {
		cout << "Found solution: \n" << endl;
		cout << fbgs.best_selection() << endl;
	}
	else {
		cerr << "Failed to find solution with flow-based greedy" << endl;
	}
//	cout << gs.selection() << endl;
}

void solve(Market& market, const std::string& filename, Price price_limit) {
	NoConstraint<Seller> seller_constr;
	NoConstraint<Selection> selection_constr;
	vector<BuyerRequest> reqs;

	if(!EntityIOUtils::loadRequests(reqs, filename)) {
		cerr << "Error loading requests" << endl;
	}

	cout << StringUtils::to_string(reqs.begin(), reqs.end(), ",") << endl;

	GreedySelector gs(market, PurchasableData::instance());
	gs.set_worst_acceptable_price(price_limit);
	if(gs.start(reqs, seller_constr, selection_constr)) {
		cout << "Found solution: \n" << endl;
		cout << gs.best_selection() << endl;
	}
	else {
		cerr << "Failed to find solution with greedy" << endl;
	}
//	cout << gs.selection() << endl;
}

}
