/*
 * PurchasableOffer.h
 *
 *  Created on: Apr 10, 2010
 *      Author: leden
 */

#ifndef PURCHASABLE_OFFER_H_
#define PURCHASABLE_OFFER_H_

#include "purchasable.h"
#include "price_utils.h"
#include "seller.h"
#include "currency.h"
#include "constraint.h"

#include <set>
#include <map>

class Seller;
class Currency;

class PurchasableOffer : public PropertiesEntity {
private:
	Price price_;
	Price shipping_;
	Seller* seller_;
	const Currency* currency_;
	std::set<Id> purchasables_;
	std::map<Id, Quantity> counter_;
	std::map< std::pair<Quantity, Quantity>, Price > wholesale_prices_;
	Quantity contents_size_;
public:
	PurchasableOffer();
	PurchasableOffer(Id id);
	PurchasableOffer(Id id, Seller& seller);
	PurchasableOffer(Id id, Seller& seller, const Currency& currency);
	Price price() const;
	Price shipping() const;
	Price wholesalePrice(Quantity quantity) const;
	void set_price(Price price);
	void set_shipping(Price price);
	void set_currency(const Currency& currency);
	void set_wholesale_price(const std::pair<Quantity, Quantity>
		quantityRangeInclusive, Price price);
	const std::map< std::pair<Quantity, Quantity>, Price >&
	wholesale_prices() const;
	const Currency& currency() const;
	void addPurchasable(const Purchasable& purchasable, Quantity quantity);
	void clearPurchasables();
	const set<Id>& purchasables() const;
	bool contains(const Purchasable& p) const;
	bool meetsConstraint(const Constraint<Purchasable>& constraint) const;
	Quantity getQuantity(const Purchasable& p) const;
	Quantity getQuantity(Id purchasable_id) const;
	Seller* seller() const;
	void set_seller(Seller* seller);
	Quantity contents_size() const;
private:
	void init(Seller* seller);
	void init(Seller* seller, const Currency* currency);

	friend bool operator<(const PurchasableOffer& a, const PurchasableOffer& b);
	friend bool operator==(const PurchasableOffer& a, const PurchasableOffer& b);
	friend ostream& operator<<(ostream& os, const PurchasableOffer& offer);
};

#endif /* PURCHASABLE_OFFER_H_ */
