/*
 * Selection.h
 *
 *  Created on: Apr 10, 2010
 *      Author: leden
 */

#ifndef SELECTION_H_
#define SELECTION_H_

#include "purchasable.h"
#include "seller.h"
#include "hash_utils.h"

#include <set>
#include <vector>
#include <map>
#include <iostream>

class Selection {
public:
	typedef std::vector< std::pair<PurchasableOffer, Quantity> >
	OfferConfiguration;
private:
	Price total_price_;
	std::map<PurchasableOffer, Quantity> offers_;
	std::set<Seller> sellers_;
	std::map<Id, Quantity> purchasable_quanities_;
public:
	Selection();
	virtual ~Selection();
	Price getTotalPrice() const;
	void set_total_price(Price price);
	const std::map<PurchasableOffer, Quantity>& offers() const;
	void clear();
	bool addOffer(const PurchasableOffer& offer, Quantity times);
	bool removeOffer(const PurchasableOffer& offer, Quantity times);
	bool clearOffer(const PurchasableOffer& offer);
	bool hasPurchasable(Id purchasable_id, Quantity minQuantity) const;
	Quantity getPurchasableQuantity(Id purchasable_id) const;
	bool hasOffer(const PurchasableOffer& offer, Quantity minQuantity) const;
	bool hasOffer(const PurchasableOffer& offer) const;
	Quantity getOfferCount(const PurchasableOffer& offer) const;
	void loadFrom(const std::vector< std::pair<PurchasableOffer, Quantity> >&
			offer_quantities);
	std::vector< std::pair<PurchasableOffer, Quantity> > toVector() const;

private:
	bool increasePurchasableCount(Id purchasable_id, int inc);
	bool decreasePurchasableCount(Id purchasable_id, int dec);

public:
	friend std::ostream& operator<<(std::ostream& os, const Selection& s);
};

#endif /* SELECTION_H_ */
