/*
 * PurchasableProvider.h
 *
 *  Created on: Apr 10, 2010
 *      Author: leden
 */

#ifndef PURCHASABLE_PROVIDER_H_
#define PURCHASABLE_PROVIDER_H_

#include "properties_entity.h"
#include "purchasable.h"
#include "purchasable_offer.h"
#include "currency.h"
#include "supply.h"
#include "constraint.h"

#include <set>
#include <vector>
#include <algorithm>

using namespace std;

class PurchasableOffer;

class Seller : public PropertiesEntity {
private:
	Price shipping_cost_;
	Price subscription_fee_;
	const Currency* default_currency_;
	map<Id, PurchasableOffer> all_offers_;
	map<Id, set<Id> > offers_;
	Supply supplies_;
public:
	Seller();
	Seller(const Currency* defaultCurrency);
	const PurchasableOffer& offer(Id offer_id) const;
	const Currency* default_currency() const;
	void set_default_currency(const Currency* currency);
	Price shipping_cost() const;
	void set_shipping_cost(Price price);
	Price subscription_fee() const;
	void set_subscription_fee(Price price);
	bool addOffer(const PurchasableOffer& offer);
	bool removeOffer(Id offer_id);
	void clearOffers();
	const Supply& supplies() const;
	void addSupply(Id purchasableId, Quantity quantity);
	void clearAllSupplies();
	void clearSupply(Id purchasable_id);
	Quantity getQuantity(Id purchasable_id) const;
	vector<const PurchasableOffer*> allOffers() const;
	bool offers(Id purchasableId) const;
	virtual set<const PurchasableOffer*> getOffers(Id purchasable_id) const;
	virtual set<const PurchasableOffer*> getOffers(
			const Constraint<Purchasable>& constraint) const;

	Quantity buyOffer(Id offer_id, Quantity times);
	Quantity returnOffer(Id offer_id, Quantity times);
	Quantity maxQuantity(Id offer_id) const;
private:
	bool hasOffer(Id offer_id);
	void init();

	friend bool operator<(const Seller& a, const Seller& b);
	friend bool operator==(const Seller& a, const Seller& b);

	friend ostream& operator<<(ostream& os, const Seller& s);
};

#endif /* PURCHASABLE_PROVIDER_H_ */
